import { useCallback, useState, useEffect, useRef } from "react";
import { useFrame } from "@react-three/fiber";
import { useMaterialStore } from "../../../../../store/simulation/useMaterialStore";
import { useProductStore } from "../../../../../store/simulation/useProductStore";
import { useSelectedProduct } from "../../../../../store/simulation/useSimulationStore";
import { useStorageUnitStore } from "../../../../../store/simulation/useStorageUnitStore";
import { useArmBotStore } from "../../../../../store/simulation/useArmBotStore";
import { useVehicleStore } from "../../../../../store/simulation/useVehicleStore";
import { usePlayButtonStore, usePauseButtonStore, useResetButtonStore } from "../../../../../store/usePlayButtonStore";

export function useRetrieveHandler() {
    const { addMaterial } = useMaterialStore();
    const { getModelUuidByActionUuid, getPointUuidByActionUuid, getEventByModelUuid } = useProductStore();
    const { getStorageUnitById, getLastMaterial, updateCurrentLoad, removeLastMaterial } = useStorageUnitStore();
    const { getVehicleById, incrementVehicleLoad, addCurrentMaterial } = useVehicleStore();
    const { selectedProduct } = useSelectedProduct();
    const { getArmBotById, addCurrentAction } = useArmBotStore();
    const { isPlaying } = usePlayButtonStore();
    const { isPaused } = usePauseButtonStore();
    const { isReset } = useResetButtonStore();

    const [activeRetrievals, setActiveRetrievals] = useState<Map<string, { action: StorageAction, isProcessing: boolean, lastCheckTime: number }>>(new Map());

    const [initialDelayComplete, setInitialDelayComplete] = useState(false);
    const delayTimerRef = useRef<NodeJS.Timeout | null>(null);

    const retrieveLogStatus = (materialUuid: string, status: string) => {
        echo.info(`${materialUuid}, ${status}`);
    }

    const createNewMaterial = useCallback((materialId: string, materialType: string, action: StorageAction) => {
        const modelUuid = getModelUuidByActionUuid(selectedProduct.productId, action.actionUuid);
        const pointUuid = getPointUuidByActionUuid(selectedProduct.productId, action.actionUuid);
        if (!modelUuid || !pointUuid) return null;
        const currentTime = performance.now();

        if (action.triggers[0]?.triggeredAsset?.triggeredModel.modelUuid &&
            action.triggers[0]?.triggeredAsset?.triggeredPoint?.pointUuid &&
            action.triggers[0]?.triggeredAsset?.triggeredAction?.actionUuid
        ) {
            const newMaterial: MaterialSchema = {
                materialId: materialId,
                materialName: `${materialType}-${Date.now()}`,
                materialType: materialType,
                isActive: false,
                isVisible: false,
                isPaused: false,
                isRendered: true,
                startTime: currentTime,
                previous: {
                    modelUuid: modelUuid,
                    pointUuid: pointUuid,
                    actionUuid: action.actionUuid
                },
                current: {
                    modelUuid: action.triggers[0].triggeredAsset.triggeredModel.modelUuid,
                    pointUuid: action.triggers[0].triggeredAsset.triggeredPoint.pointUuid,
                    actionUuid: action.triggers[0].triggeredAsset.triggeredAction.actionUuid
                },
            };

            addMaterial(newMaterial);
            return newMaterial;
        }
        return null;
    }, [addMaterial, getModelUuidByActionUuid, getPointUuidByActionUuid, selectedProduct.productId]);

    useEffect(() => {
        if (isPlaying && !initialDelayComplete) {
            delayTimerRef.current = setTimeout(() => {
                setInitialDelayComplete(true);
            }, 3000);
        }

        return () => {
            if (delayTimerRef.current) {
                clearTimeout(delayTimerRef.current);
            }
        };
    }, [isPlaying, initialDelayComplete]);

    useEffect(() => {
        if (isReset || isPaused) {
            setInitialDelayComplete(false);
            if (delayTimerRef.current) {
                clearTimeout(delayTimerRef.current);
            }
        }
    }, [isReset, isPaused]);

    useFrame(() => {
        if (!isPlaying || isPaused || isReset || !initialDelayComplete) return;

        const currentTime = performance.now();
        const completedActions: string[] = [];
        let hasChanges = false;

        activeRetrievals.forEach((retrieval, actionUuid) => {
            if (retrieval.isProcessing) return;

            const storageUnit = getStorageUnitById(
                getModelUuidByActionUuid(selectedProduct.productId, retrieval.action.actionUuid) ?? ''
            );

            if (!storageUnit || storageUnit.currentLoad <= 0) {
                completedActions.push(actionUuid);
                hasChanges = true;
                return;
            }

            if (retrieval.action.triggers.length === 0 || !retrieval.action.triggers[0].triggeredAsset) {
                return;
            }

            const triggeredModel = getEventByModelUuid(
                selectedProduct.productId,
                retrieval.action.triggers[0].triggeredAsset.triggeredModel.modelUuid
            );

            if (!triggeredModel) return;

            let isIdle = false;

            if (triggeredModel.type === 'roboticArm') {
                const armBot = getArmBotById(triggeredModel.modelUuid);
                isIdle = (armBot && !armBot.isActive && armBot.state === 'idle' && !armBot.currentAction) || false;

                if (isIdle) {
                    setActiveRetrievals(prev => {
                        const newRetrievals = new Map(prev);
                        newRetrievals.set(actionUuid, {
                            ...retrieval,
                            isProcessing: true,
                            lastCheckTime: currentTime
                        });
                        return newRetrievals;
                    });

                    const lastMaterial = getLastMaterial(storageUnit.modelUuid);
                    if (lastMaterial) {
                        const material = createNewMaterial(
                            lastMaterial.materialId,
                            lastMaterial.materialType,
                            storageUnit.point.action
                        );

                        if (material) {
                            addCurrentAction(
                                triggeredModel.modelUuid,
                                retrieval.action.triggers[0].triggeredAsset.triggeredAction?.actionUuid ?? '',
                                material.materialType,
                                material.materialId
                            );
                            retrieveLogStatus(material.materialName, `is being picked by ${armBot?.modelName}`);
                        }
                    }

                    setActiveRetrievals(prev => {
                        const newRetrievals = new Map(prev);
                        newRetrievals.set(actionUuid, {
                            ...retrieval,
                            isProcessing: false,
                            lastCheckTime: currentTime
                        });
                        return newRetrievals;
                    });
                }

            } else if (triggeredModel.type === 'vehicle') {
                const vehicle = getVehicleById(triggeredModel.modelUuid);
                isIdle = (vehicle && !vehicle.isActive && vehicle.state === 'idle' && vehicle.isPicking) || false;

                if (!vehicle) return;
                // const loadDuration = vehicle.point.action.unLoadDuration;

                if (isIdle) {
                    setActiveRetrievals(prev => {
                        const newRetrievals = new Map(prev);
                        newRetrievals.set(actionUuid, {
                            ...retrieval,
                            isProcessing: true,
                            lastCheckTime: currentTime
                        });
                        return newRetrievals;
                    });

                    const lastMaterial = getLastMaterial(storageUnit.modelUuid);
                    if (lastMaterial) {
                        if (vehicle?.currentLoad < vehicle.point.action.loadCapacity) {
                            const material = createNewMaterial(
                                lastMaterial.materialId,
                                lastMaterial.materialType,
                                storageUnit.point.action
                            );

                            if (material) {
                                removeLastMaterial(storageUnit.modelUuid);
                                updateCurrentLoad(storageUnit.modelUuid, -1)
                                incrementVehicleLoad(vehicle.modelUuid, 1);
                                addCurrentMaterial(vehicle.modelUuid, material.materialType, material.materialId);
                                retrieveLogStatus(material.materialName, `is picked by ${vehicle.modelName}`);
                            }
                        }
                    }

                    setActiveRetrievals(prev => {
                        const newRetrievals = new Map(prev);
                        newRetrievals.set(actionUuid, {
                            ...retrieval,
                            isProcessing: false,
                            lastCheckTime: currentTime
                        });
                        return newRetrievals;
                    });
                }

            }
        });

        if (hasChanges || completedActions.length > 0) {
            setActiveRetrievals(prev => {
                const newRetrievals = new Map(prev);
                completedActions.forEach(actionUuid => newRetrievals.delete(actionUuid));
                return newRetrievals;
            });
        }
    });

    const handleRetrieve = useCallback((action: StorageAction) => {
        if (!action || action.actionType !== 'retrieve') return;

        setActiveRetrievals(prev => {
            const newRetrievals = new Map(prev);
            newRetrievals.set(action.actionUuid, {
                action,
                isProcessing: false,
                lastCheckTime: performance.now()
            });
            return newRetrievals;
        });
    }, []);

    useEffect(() => {
        if (isReset) {
            setActiveRetrievals(new Map());
            setInitialDelayComplete(false);
        }
    }, [isReset]);

    return {
        handleRetrieve,
    };
}